@@ -182,6 +182,9 @@ public:
 	// Callback must return FZ_REPLY_CONTINUE or FZ_REPLY_ERROR
 	std::function<int(unsigned char const* data, unsigned int len)> on_data_;
 
+	// Called if !success && got_body
+	std::function<int(unsigned char const* data, unsigned int len)> on_error_data_;
+
 	bool success() const {
 		return code_ >= 200 && code_ < 300;
 	}
@@ -260,7 +263,7 @@ protected:
 	CTlsSocket* m_pTlsSocket{};
 
 	virtual void OnConnect() override;
-	virtual void OnClose(int error) override;
+	virtual void OnSocketError(int error) override;
 	virtual void OnReceive() override;
 	virtual int OnSend() override;
 	
