@@ -144,8 +144,6 @@ CTlsSocketImpl::CTlsSocketImpl(CTlsSocket& tlsSocket, fz::socket& socket, CContr
 	, m_socket(socket)
 	, socketBackend_(std::make_unique<CSocketBackend>(static_cast<fz::event_handler*>(&tlsSocket_), m_socket, m_pOwner->GetEngine().GetRateLimiter()))
 {
-	m_implicitTrustedCert.data = nullptr;
-	m_implicitTrustedCert.size = 0;
 }
 
 CTlsSocketImpl::~CTlsSocketImpl()
@@ -156,36 +154,38 @@ CTlsSocketImpl::~CTlsSocketImpl()
 bool CTlsSocketImpl::Init()
 {
 	// This function initializes GnuTLS
-	m_initialized = true;
-	int res = gnutls_global_init();
-	if (res) {
-		LogError(res, L"gnutls_global_init");
-		Uninit();
-		return false;
-	}
+	if (!m_initialized) {
+		m_initialized = true;
+		int res = gnutls_global_init();
+		if (res) {
+			LogError(res, L"gnutls_global_init");
+			Uninit();
+			return false;
+		}
 
 #if TLSDEBUG
-	if (!pLoggingControlSocket) {
-		pLoggingControlSocket = m_pOwner;
-		gnutls_global_set_log_function(log_func);
-		gnutls_global_set_log_level(99);
-	}
+		if (!pLoggingControlSocket) {
+			pLoggingControlSocket = m_pOwner;
+			gnutls_global_set_log_function(log_func);
+			gnutls_global_set_log_level(99);
+		}
 #endif
-	res = gnutls_certificate_allocate_credentials(&m_certCredentials);
-	if (res < 0) {
-		LogError(res, L"gnutls_certificate_allocate_credentials");
-		Uninit();
-		return false;
+	}
+
+	if (!m_certCredentials) {
+		int res = gnutls_certificate_allocate_credentials(&m_certCredentials);
+		if (res < 0) {
+			LogError(res, L"gnutls_certificate_allocate_credentials");
+			Uninit();
+			return false;
+		}
 	}
 
 	if (!InitSession()) {
 		return false;
 	}
 
-	m_shutdown_requested = false;
-
 	// At this point, we can start shaking hands.
-
 	return true;
 }
 
@@ -288,6 +288,10 @@ void CTlsSocketImpl::UninitSession()
 
 void CTlsSocketImpl::LogError(int code, std::wstring const& function, MessageType logLevel)
 {
+	if (logLevel < MessageType::Debug_Warning && m_tlsState == CTlsSocket::TlsState::closed && shutdown_silence_read_errors_) {
+		logLevel = MessageType::Debug_Warning;
+	}
+
 	if (code == GNUTLS_E_WARNING_ALERT_RECEIVED || code == GNUTLS_E_FATAL_ALERT_RECEIVED) {
 		PrintAlert(logLevel);
 	}
@@ -458,21 +462,19 @@ void CTlsSocketImpl::OnRead()
 		return;
 	}
 
-	const int direction = gnutls_record_get_direction(m_session);
-	if (direction && !m_lastReadFailed) {
-		m_pOwner->LogMessage(MessageType::Debug_Debug, L"CTlsSocketImpl::Postponing read");
-		return;
-	}
-
 	if (m_tlsState == CTlsSocket::TlsState::handshake) {
+		int const direction = gnutls_record_get_direction(m_session);
+		if (direction) {
+			m_pOwner->LogMessage(MessageType::Debug_Debug, L"CTlsSocketImpl::Postponing read");
+			return;
+		}
 		ContinueHandshake();
 	}
-	if (m_tlsState == CTlsSocket::TlsState::closing) {
-		ContinueShutdown();
-	}
-	else if (m_tlsState == CTlsSocket::TlsState::conn) {
-		CheckResumeFailedReadWrite();
-		TriggerEvents();
+	else if (m_tlsState >= CTlsSocket::TlsState::conn) {
+		if (m_lastReadFailed) {
+			m_lastReadFailed = false;
+			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, 0);
+		}
 	}
 }
 
@@ -486,20 +488,41 @@ void CTlsSocketImpl::OnSend()
 		return;
 	}
 
-	const int direction = gnutls_record_get_direction(m_session);
-	if (!direction && !m_lastWriteFailed) {
-		return;
-	}
-
 	if (m_tlsState == CTlsSocket::TlsState::handshake) {
+		int const direction = gnutls_record_get_direction(m_session);
+		if (!direction) {
+			return;
+		}
 		ContinueHandshake();
 	}
 	else if (m_tlsState == CTlsSocket::TlsState::closing) {
 		ContinueShutdown();
 	}
 	else if (m_tlsState == CTlsSocket::TlsState::conn) {
-		CheckResumeFailedReadWrite();
-		TriggerEvents();
+		ContinueWrite();
+	}
+}
+
+void CTlsSocketImpl::ContinueWrite()
+{
+	if (m_lastWriteFailed) {
+		ssize_t res = GNUTLS_E_AGAIN;
+		while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket) {
+			res = gnutls_record_send(m_session, nullptr, 0);
+		}
+
+		if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) {
+			return;
+		}
+
+		if (res < 0) {
+			Failure(static_cast<int>(res), true);
+			return;
+		}
+
+		m_writeSkip += static_cast<int>(res);
+		m_lastWriteFailed = false;
+		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::write, 0);
 	}
 }
 
@@ -536,8 +559,13 @@ bool CTlsSocketImpl::ResumedSession() const
 int CTlsSocketImpl::Handshake(const CTlsSocketImpl* pPrimarySocket, bool try_resume)
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, L"CTlsSocketImpl::Handshake()");
-	if (!m_session) {
-		m_pOwner->LogMessage(MessageType::Debug_Warning, L"Called CTlsSocketImpl::Handshake without session");
+
+	if (m_tlsState != CTlsSocket::TlsState::noconn) {
+		m_pOwner->LogMessage(MessageType::Debug_Warning, L"Called CTlsSocketImpl::Handshake on a socket that isn't idle");
+		return FZ_REPLY_ERROR;
+	}
+
+	if (!Init()) {
 		return FZ_REPLY_ERROR;
 	}
 
@@ -546,6 +574,7 @@ int CTlsSocketImpl::Handshake(const CTlsSocketImpl* pPrimarySocket, bool try_res
 	if (pPrimarySocket) {
 		if (!pPrimarySocket->m_session) {
 			m_pOwner->LogMessage(MessageType::Debug_Warning, L"Primary socket has no session");
+			Uninit();
 			return FZ_REPLY_ERROR;
 		}
 
@@ -566,25 +595,19 @@ int CTlsSocketImpl::Handshake(const CTlsSocketImpl* pPrimarySocket, bool try_res
 		}
 
 		hostname_ = pPrimarySocket->m_socket.peer_host();
-
-		int port, tmp;
-		port = m_socket.remote_port(tmp);
-		if (port <= 0) {
-			return FZ_REPLY_ERROR;
-		}
-		port_ = port;
 	}
 	else {
 		hostname_ = m_socket.peer_host();
-
-		int port, tmp;
-		port = m_socket.remote_port(tmp);
-		if (port <= 0) {
-			return FZ_REPLY_ERROR;
-		}
-		port_ = port;
 	}
 
+	int port, tmp;
+	port = m_socket.remote_port(tmp);
+	if (port <= 0) {
+		Uninit();
+		return FZ_REPLY_ERROR;
+	}
+	port_ = port;
+
 	if (!hostname_.empty() && fz::get_address_type(hostname_) == fz::address_type::unknown) {
 		auto const utf8 = fz::to_utf8(hostname_);
 		if (!utf8.empty()) {
@@ -634,8 +657,8 @@ int CTlsSocketImpl::ContinueHandshake()
 			return res;
 		}
 
-		if (m_shutdown_requested) {
-			int error = Shutdown();
+		if (shutdown_requested_) {
+			int error = Shutdown(shutdown_silence_read_errors_);
 			if (!error || error != EAGAIN) {
 				tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, error);
 			}
@@ -658,7 +681,7 @@ int CTlsSocketImpl::Read(void *buffer, unsigned int len, int& error)
 		error = EAGAIN;
 		return -1;
 	}
-	else if (m_tlsState != CTlsSocket::TlsState::conn) {
+	else if (m_tlsState == CTlsSocket::TlsState::noconn) {
 		error = ENOTCONN;
 		return -1;
 	}
@@ -671,19 +694,13 @@ int CTlsSocketImpl::Read(void *buffer, unsigned int len, int& error)
 		auto min = std::min(static_cast<size_t>(len), peekBuffer_.size());
 		memcpy(buffer, peekBuffer_.get(), min);
 		peekBuffer_.consume(min);
-
-		TriggerEvents();
-
 		error = 0;
 		return static_cast<int>(min);
 	}
 
 	int res = DoCallGnutlsRecordRecv(buffer, len);
 	if (res >= 0) {
-		if (res > 0) {
-			TriggerEvents();
-		}
-		else {
+		if (!res) {
 			// Peer did already initiate a shutdown, reply to it
 			gnutls_bye(m_session, GNUTLS_SHUT_WR);
 			// Note: Theoretically this could return a write error.
@@ -691,7 +708,6 @@ int CTlsSocketImpl::Read(void *buffer, unsigned int len, int& error)
 			// to close the connection after sending its shutdown
 			// notification.
 		}
-
 		error = 0;
 		return res;
 	}
@@ -749,8 +765,6 @@ int CTlsSocketImpl::Write(const void *buffer, unsigned int len, int& error)
 		error = 0;
 		int written = static_cast<int>(res) + m_writeSkip;
 		m_writeSkip = 0;
-
-		TriggerEvents();
 		return written;
 	}
 
@@ -774,60 +788,6 @@ int CTlsSocketImpl::Write(const void *buffer, unsigned int len, int& error)
 	}
 }
 
-void CTlsSocketImpl::TriggerEvents()
-{
-	if (m_tlsState != CTlsSocket::TlsState::conn) {
-		return;
-	}
-
-	if (m_canTriggerRead) {
-		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, 0);
-		m_canTriggerRead = false;
-	}
-
-	if (m_canTriggerWrite) {
-		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::write, 0);
-		m_canTriggerWrite = false;
-	}
-}
-
-void CTlsSocketImpl::CheckResumeFailedReadWrite()
-{
-	if (m_lastWriteFailed) {
-		ssize_t res = GNUTLS_E_AGAIN;
-		while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket) {
-			res = gnutls_record_send(m_session, nullptr, 0);
-		}
-
-		if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) {
-			return;
-		}
-
-		if (res < 0) {
-			Failure(static_cast<int>(res), true);
-			return;
-		}
-
-		m_writeSkip += static_cast<int>(res);
-		m_lastWriteFailed = false;
-		m_canTriggerWrite = true;
-	}
-	if (m_lastReadFailed) {
-		int res = DoCallGnutlsRecordRecv(peekBuffer_.get(65536), 65536);
-		if (res < 0) {
-			if (res != GNUTLS_E_INTERRUPTED && res != GNUTLS_E_AGAIN) {
-				Failure(res, true);
-			}
-			return;
-		}
-
-		peekBuffer_.add(static_cast<size_t>(res));
-
-		m_lastReadFailed = false;
-		m_canTriggerRead = true;
-	}
-}
-
 void CTlsSocketImpl::Failure(int code, bool send_close, std::wstring const& function)
 {
 	m_pOwner->LogMessage(MessageType::Debug_Debug, L"CTlsSocketImpl::Failure(%d)", code);
@@ -840,7 +800,9 @@ void CTlsSocketImpl::Failure(int code, bool send_close, std::wstring const& func
 #endif
 				)
 			{
-				m_pOwner->LogMessage(MessageType::Status, _("Server did not properly shut down TLS connection"));
+				if (m_tlsState != CTlsSocket::TlsState::closed || !shutdown_silence_read_errors_) {
+					m_pOwner->LogMessage(MessageType::Status, _("Server did not properly shut down TLS connection"));
+				}
 			}
 		}
 	}
@@ -883,7 +845,7 @@ int CTlsSocketImpl::Peek(void *buffer, unsigned int len, int& error)
 	return read;
 }
 
-int CTlsSocketImpl::Shutdown()
+int CTlsSocketImpl::Shutdown(bool silenceReadErrors)
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, L"CTlsSocketImpl::Shutdown()");
 
@@ -898,7 +860,8 @@ int CTlsSocketImpl::Shutdown()
 	if (m_tlsState == CTlsSocket::TlsState::handshake || m_tlsState == CTlsSocket::TlsState::verifycert) {
 		// Shutdown during handshake is not a good idea.
 		m_pOwner->LogMessage(MessageType::Debug_Verbose, L"Shutdown during handshake, postponing");
-		m_shutdown_requested = true;
+		shutdown_requested_ = true;
+		shutdown_silence_read_errors_ = silenceReadErrors;
 		return EAGAIN;
 	}
 
@@ -907,6 +870,7 @@ int CTlsSocketImpl::Shutdown()
 	}
 
 	m_tlsState = CTlsSocket::TlsState::closing;
+	shutdown_silence_read_errors_ = silenceReadErrors;
 
 	int res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
 	while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket) {
@@ -955,17 +919,11 @@ void CTlsSocketImpl::TrustCurrentCert(bool trusted)
 
 	if (trusted) {
 		m_tlsState = CTlsSocket::TlsState::conn;
-
-		if (m_lastWriteFailed) {
-			m_lastWriteFailed = false;
+		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::connection, 0);
+		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, 0);
+		if (!shutdown_requested_) {
+			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::write, 0);
 		}
-		CheckResumeFailedReadWrite();
-
-		if (m_tlsState == CTlsSocket::TlsState::conn) {
-			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::connection, 0);
-		}
-
-		TriggerEvents();
 
 		return;
 	}
