@@ -21,7 +21,7 @@ public:
 
 	virtual bool CanStartComparison();
 	virtual void StartComparison();
-	virtual bool get_next_file(wxString& name, bool &dir, int64_t &size, fz::datetime& date);
+	virtual bool get_next_file(std::wstring & name, bool &dir, int64_t &size, fz::datetime& date) override;
 	virtual void FinishComparison();
 	virtual void OnExitComparisonMode();
 
@@ -44,12 +44,12 @@ protected:
 	virtual wxString GetItemText(int item, unsigned int column);
 
 	// Clears all selections and returns the list of items that were selected
-	std::list<wxString> RememberSelectedItems(wxString& focused);
+	std::vector<std::wstring> RememberSelectedItems(std::wstring & focused, int & focusItem);
 
 	// Select a list of items based in their names.
 	// Sort order may not change between call to RememberSelectedItems and
 	// ReselectItems
-	void ReselectItems(std::list<wxString>& selectedNames, wxString focused, bool ensureVisible = false);
+	void ReselectItems(std::vector<std::wstring>& selectedNames, std::wstring focused, int focusedItem, bool ensureVisible = false);
 
 
 	// Declared const due to design error in wxWidgets.
@@ -67,7 +67,7 @@ protected:
 
 	virtual std::unique_ptr<CFileListCtrlSortBase> GetSortComparisonObject() override;
 
-	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(t_statechange_notifications notification, std::wstring const& data, const void* data2) override;
 	void ApplyCurrentFilter();
 	void SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
 	bool UpdateDirectoryListing(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
@@ -108,9 +108,9 @@ protected:
 	// already.
 	struct t_linkResolveState
 	{
-		ServerWithCredentials server;
+		Site site;
 		CServerPath remote_path;
-		wxString link;
+		std::wstring link;
 		CLocalPath local_path;
 	};
 	std::unique_ptr<t_linkResolveState> m_pLinkResolveState;
