@@ -9,6 +9,7 @@
 #include "window_state_manager.h"
 #include "xrc_helper.h"
 
+#include <libfilezilla/file.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
 class CChangedFileDialog : public wxDialogEx
@@ -92,18 +93,17 @@ void CEditHandler::RemoveTemporaryFiles(wxString const& temp)
 			continue;
 		}
 
-		RemoveTemporaryFilesInSpecificDir(temp + file + sep);
+		RemoveTemporaryFilesInSpecificDir((temp + file + sep).ToStdWstring());
 	} while (dir.GetNext(&file));
 }
 
-void CEditHandler::RemoveTemporaryFilesInSpecificDir(wxString const& temp)
+void CEditHandler::RemoveTemporaryFilesInSpecificDir(std::wstring const& temp)
 {
-	const wxString lockfile = temp + _("fz3temp-lockfile");
+	std::wstring const lockfile = temp + L"fz3temp-lockfile";
 	if (wxFileName::FileExists(lockfile)) {
 #ifndef __WXMSW__
-		int fd = open(lockfile.mb_str(), O_RDWR | O_CLOEXEC, 0);
-		if (fd >= 0)
-		{
+		int fd = open(fz::to_string(lockfile).c_str(), O_RDWR | O_CLOEXEC, 0);
+		if (fd >= 0) {
 			// Try to lock 1 byte region in the lockfile. m_type specifies the byte to lock.
 			struct flock f = {};
 			f.l_type = F_WRLCK;
@@ -119,12 +119,11 @@ void CEditHandler::RemoveTemporaryFilesInSpecificDir(wxString const& temp)
 			close(fd);
 		}
 #endif
-		{
-			wxLogNull log;
-			wxRemoveFile(lockfile);
-		}
-		if (wxFileName::FileExists(lockfile))
+		fz::remove_file(fz::to_native(lockfile));
+
+		if (wxFileName::FileExists(lockfile)) {
 			return;
+		}
 	}
 
 	wxLogNull log;
@@ -153,11 +152,13 @@ std::wstring CEditHandler::GetLocalDirectory()
 	// which will cause problems when calculating maximum allowed file
 	// length
 	wxString dir = tmpdir.GetLongPath();
-	if (dir.empty() || !wxFileName::DirExists(dir))
+	if (dir.empty() || !wxFileName::DirExists(dir)) {
 		return std::wstring();
+	}
 
-	if (dir.Last() != wxFileName::GetPathSeparator())
+	if (dir.Last() != wxFileName::GetPathSeparator()) {
 		dir += wxFileName::GetPathSeparator();
+	}
 
 	// On POSIX, the permissions of the created directory (700) ensure
 	// that this is a safe operation.
@@ -250,9 +251,9 @@ CEditHandler::fileState CEditHandler::GetFileState(wxString const& fileName) con
 	return iter->state;
 }
 
-CEditHandler::fileState CEditHandler::GetFileState(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server) const
+CEditHandler::fileState CEditHandler::GetFileState(wxString const& fileName, CServerPath const& remotePath, Site const& site) const
 {
-	std::list<t_fileData>::const_iterator iter = GetFile(fileName, remotePath, server);
+	std::list<t_fileData>::const_iterator iter = GetFile(fileName, remotePath, site);
 	if (iter == m_fileDataList[remote].end()) {
 		return unknown;
 	}
@@ -260,11 +261,11 @@ CEditHandler::fileState CEditHandler::GetFileState(wxString const& fileName, CSe
 	return iter->state;
 }
 
-int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileState state, ServerWithCredentials const& server) const
+int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileState state, Site const& site) const
 {
 	int count = 0;
 	if (state == unknown) {
-		wxASSERT(!server);
+		wxASSERT(!site);
 		if (type != remote) {
 			count += m_fileDataList[local].size();
 		}
@@ -273,14 +274,14 @@ int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileSt
 		}
 	}
 	else {
-		auto f = [state, &server](decltype(m_fileDataList[0]) & items) {
+		auto f = [state, &site](decltype(m_fileDataList[0]) & items) {
 			int cnt = 0;
 			for (auto const& data : items) {
 				if (data.state != state) {
 					continue;
 				}
 
-				if (!server || data.server == server) {
+				if (!site || data.site == site) {
 					++cnt;
 				}
 			}
@@ -297,7 +298,7 @@ int CEditHandler::GetFileCount(CEditHandler::fileType type, CEditHandler::fileSt
 	return count;
 }
 
-bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
+bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring& fileName, CServerPath const& remotePath, Site const& site)
 {
 	wxASSERT(type != none);
 
@@ -306,7 +307,7 @@ bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring& fileName,
 		state = GetFileState(fileName);
 	}
 	else {
-		state = GetFileState(fileName, remotePath, server);
+		state = GetFileState(fileName, remotePath, site);
 	}
 	if (state != unknown) {
 		wxFAIL_MSG(_T("File state not unknown"));
@@ -322,17 +323,19 @@ bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring& fileName,
 	}
 	else {
 		data.file = fileName;
-		data.name = wxFileName(fileName).GetFullName();
+		data.name = wxFileName(fileName).GetFullName().ToStdWstring();
 		data.state = edit;
 	}
 	data.remotePath = remotePath;
-	data.server = server;
+	data.site = site;
 
-	if (type == local && !COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL))
+	if (type == local && !COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL)) {
 		return StartEditing(local, data);
+	}
 
-	if (type == remote || StartEditing(type, data))
+	if (type == remote || StartEditing(type, data)) {
 		m_fileDataList[type].push_back(data);
+	}
 
 	return true;
 }
@@ -340,21 +343,23 @@ bool CEditHandler::AddFile(CEditHandler::fileType type, std::wstring& fileName,
 bool CEditHandler::Remove(const wxString& fileName)
 {
 	std::list<t_fileData>::iterator iter = GetFile(fileName);
-	if (iter == m_fileDataList[local].end())
+	if (iter == m_fileDataList[local].end()) {
 		return true;
+	}
 
 	wxASSERT(iter->state != upload && iter->state != upload_and_remove);
-	if (iter->state == upload || iter->state == upload_and_remove)
+	if (iter->state == upload || iter->state == upload_and_remove) {
 		return false;
+	}
 
 	m_fileDataList[local].erase(iter);
 
 	return true;
 }
 
-bool CEditHandler::Remove(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
+bool CEditHandler::Remove(wxString const& fileName, CServerPath const& remotePath, Site const& site)
 {
-	std::list<t_fileData>::iterator iter = GetFile(fileName, remotePath, server);
+	std::list<t_fileData>::iterator iter = GetFile(fileName, remotePath, site);
 	if (iter == m_fileDataList[remote].end()) {
 		return true;
 	}
@@ -412,7 +417,7 @@ bool CEditHandler::RemoveAll(bool force)
 	return m_fileDataList[local].empty() && m_fileDataList[remote].empty();
 }
 
-bool CEditHandler::RemoveAll(fileState state, ServerWithCredentials const& server)
+bool CEditHandler::RemoveAll(fileState state, Site const& site)
 {
 	// Others not implemented
 	wxASSERT(state == upload_and_remove_failed);
@@ -428,7 +433,7 @@ bool CEditHandler::RemoveAll(fileState state, ServerWithCredentials const& serve
 			continue;
 		}
 
-		if (server && iter->server != server) {
+		if (site && iter->site != site) {
 			keep.push_back(*iter);
 			continue;
 		}
@@ -470,7 +475,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 	return iter;
 }
 
-std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
+std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(wxString const& fileName, CServerPath const& remotePath, Site const& site)
 {
 	std::list<t_fileData>::iterator iter;
 	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
@@ -478,7 +483,7 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(wxString con
 			continue;
 		}
 
-		if (iter->server != server) {
+		if (iter->site != site) {
 			continue;
 		}
 
@@ -492,7 +497,7 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(wxString con
 	return iter;
 }
 
-std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server) const
+std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(wxString const& fileName, CServerPath const& remotePath, Site const& site) const
 {
 	std::list<t_fileData>::const_iterator iter;
 	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
@@ -500,7 +505,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(wxStri
 			continue;
 		}
 
-		if (iter->server != server) {
+		if (iter->site != site) {
 			continue;
 		}
 
@@ -543,9 +548,9 @@ void CEditHandler::FinishTransfer(bool, wxString const& fileName)
 	SetTimerState();
 }
 
-void CEditHandler::FinishTransfer(bool successful, wxString const& fileName, CServerPath const& remotePath, ServerWithCredentials const& server)
+void CEditHandler::FinishTransfer(bool successful, wxString const& fileName, CServerPath const& remotePath, Site const& site)
 {
-	auto iter = GetFile(fileName, remotePath, server);
+	auto iter = GetFile(fileName, remotePath, site);
 	if (iter == m_fileDataList[remote].end()) {
 		return;
 	}
@@ -611,9 +616,9 @@ bool CEditHandler::StartEditing(wxString const& file)
 	return StartEditing(local, *iter);
 }
 
-bool CEditHandler::StartEditing(wxString const& file, CServerPath const& remotePath, ServerWithCredentials const& server)
+bool CEditHandler::StartEditing(wxString const& file, CServerPath const& remotePath, Site const& site)
 {
-	auto iter = GetFile(file, remotePath, server);
+	auto iter = GetFile(file, remotePath, site);
 	if (iter == m_fileDataList[remote].end()) {
 		return false;
 	}
@@ -627,16 +632,19 @@ bool CEditHandler::StartEditing(CEditHandler::fileType type, t_fileData& data)
 	wxASSERT(data.state == edit);
 
 	bool is_link;
-	if (fz::local_filesys::get_file_info(fz::to_native(data.file), is_link, 0, &data.modificationTime, 0) != fz::local_filesys::file)
+	if (fz::local_filesys::get_file_info(fz::to_native(data.file), is_link, 0, &data.modificationTime, 0) != fz::local_filesys::file) {
 		return false;
+	}
 
 	bool program_exists = false;
 	wxString cmd = GetOpenCommand(data.file, program_exists);
-	if (cmd.empty() || !program_exists)
+	if (cmd.empty() || !program_exists) {
 		return false;
+	}
 
-	if (!wxExecute(cmd))
+	if (!wxExecute(cmd)) {
 		return false;
+	}
 
 	return true;
 }
@@ -758,7 +766,7 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 			dlg.SetChildLabel(XRCID("ID_OPENEDAS"), file);
 	}
 
-	dlg.SetChildLabel(XRCID("ID_SERVER"), iter->server.Format(ServerFormat::with_user_and_optional_port));
+	dlg.SetChildLabel(XRCID("ID_SERVER"), iter->site.Format(ServerFormat::with_user_and_optional_port));
 	dlg.SetChildLabel(XRCID("ID_REMOTEPATH"), iter->remotePath.GetPath());
 
 	dlg.GetSizer()->Fit(&dlg);
@@ -770,9 +778,9 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 	return res;
 }
 
-bool CEditHandler::UploadFile(wxString const& file, CServerPath const& remotePath, ServerWithCredentials const& server, bool unedit)
+bool CEditHandler::UploadFile(wxString const& file, CServerPath const& remotePath, Site const& site, bool unedit)
 {
-	std::list<t_fileData>::iterator iter = GetFile(file, remotePath, server);
+	std::list<t_fileData>::iterator iter = GetFile(file, remotePath, site);
 	return UploadFile(remote, iter, unedit);
 }
 
@@ -821,7 +829,7 @@ bool CEditHandler::UploadFile(fileType type, std::list<t_fileData>::iterator ite
 		return false;
 	}
 
-	m_pQueue->QueueFile(false, false, file, iter->name, localPath, iter->remotePath, iter->server, size, type, QueuePriority::high);
+	m_pQueue->QueueFile(false, false, file, iter->name, localPath, iter->remotePath, iter->site, size, type, QueuePriority::high);
 	m_pQueue->QueueFile_Finish(true);
 
 	return true;
@@ -944,32 +952,35 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 	wxString associations = COptions::Get()->GetOption(OPTION_EDIT_CUSTOMASSOCIATIONS) + _T("\n");
 	associations.Replace(_T("\r"), _T(""));
 	int pos;
-	while ((pos = associations.Find('\n')) != -1)
-	{
+	while ((pos = associations.Find('\n')) != -1) {
 		wxString assoc = associations.Left(pos);
 		associations = associations.Mid(pos + 1);
 
-		if (assoc.empty())
+		if (assoc.empty()) {
 			continue;
+		}
 
 		wxString command;
-		if (!UnquoteCommand(assoc, command))
+		if (!UnquoteCommand(assoc, command)) {
 			continue;
+		}
 
-		if (assoc != ext)
+		if (assoc != ext) {
 			continue;
+		}
 
 		wxString prog = command;
 
 		wxString args;
-		if (!UnquoteCommand(prog, args))
+		if (!UnquoteCommand(prog, args)) {
 			return wxString();
+		}
 
-		if (prog.empty())
+		if (prog.empty()) {
 			return wxString();
+		}
 
-		if (!ProgramExists(prog))
-		{
+		if (!ProgramExists(prog)) {
 			program_exists = false;
 			return prog;
 		}
@@ -992,12 +1003,12 @@ std::wstring CEditHandler::GetTemporaryFile(std::wstring name)
 #ifdef __WXMSW__
 	// MAX_PATH - 1 is theoretical limit, we subtract another 4 to allow
 	// editors which create temporary files
-	int max = MAX_PATH - 5;
+	size_t max = MAX_PATH - 5;
 #else
-	int max = -1;
+	size_t max = std::wstring::npos;
 #endif
-	if (max != -1) {
-		name = TruncateFilename(m_localDir, name, max).ToStdWstring();
+	if (max != std::wstring::npos) {
+		name = TruncateFilename(m_localDir, name, max);
 		if (name.empty()) {
 			return std::wstring();
 		}
@@ -1008,16 +1019,16 @@ std::wstring CEditHandler::GetTemporaryFile(std::wstring name)
 		return file;
 	}
 
-	if (max != -1) {
-		max--;
+	if (max != std::wstring::npos) {
+		--max;
 	}
 	int cutoff = 1;
 	int n = 1;
 	while (++n < 10000) { // Just to give up eventually
 		// Further reduce length if needed
-		if (max != -1 && n >= cutoff) {
+		if (max != std::wstring::npos && n >= cutoff) {
 			cutoff *= 10;
-			max--;
+			--max;
 			name = TruncateFilename(m_localDir, name, max);
 			if (name.empty()) {
 				return std::wstring();
@@ -1040,23 +1051,22 @@ std::wstring CEditHandler::GetTemporaryFile(std::wstring name)
 	return std::wstring();
 }
 
-wxString CEditHandler::TruncateFilename(const wxString path, const wxString& name, int max)
+std::wstring CEditHandler::TruncateFilename(std::wstring const& path, std::wstring const& name, size_t max)
 {
-	const int pathlen = path.Len();
-	const int namelen = name.Len();
-	if (namelen + pathlen > max)
-	{
-		int pos = name.Find('.', true);
-		if (pos != -1)
-		{
-			int extlen = namelen - pos;
+	size_t const pathlen = path.size();
+	size_t const namelen = name.size();
+
+	if (namelen + pathlen > max) {
+		size_t pos = name.rfind('.');
+		if (pos != std::wstring::npos) {
+			size_t extlen = namelen - pos;
 			if (pathlen + extlen >= max)
 			{
 				// Cannot truncate extension
-				return wxString();
+				return std::wstring();
 			}
 
-			return name.Left(max - pathlen - extlen) + name.Mid(pos);
+			return name.substr(0, max - pathlen - extlen) + name.substr(pos);
 		}
 	}
 
@@ -1169,7 +1179,7 @@ int CEditHandlerStatusDialog::ShowModal()
 				pListCtrl->SetItem(i, COLUMN_STATUS, _("Unknown"));
 				break;
 			}
-			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
+			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->site.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
 			CEditHandler::t_fileData* pData = new CEditHandler::t_fileData(*iter);
 			pListCtrl->SetItemPtrData(i, (wxUIntPtr)pData);
 		}
@@ -1197,7 +1207,7 @@ int CEditHandlerStatusDialog::ShowModal()
 				pListCtrl->SetItem(i, COLUMN_STATUS, _("Unknown"));
 				break;
 			}
-			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->server.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
+			pListCtrl->SetItem(i, COLUMN_REMOTEPATH, iter->site.Format(ServerFormat::with_user_and_optional_port) + iter->remotePath.GetPath());
 			CEditHandler::t_fileData* pData = new CEditHandler::t_fileData(*iter);
 			pListCtrl->SetItemPtrData(i, (wxUIntPtr)pData);
 		}
@@ -1292,7 +1302,7 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent&)
 			pListCtrl->DeleteItem(i);
 		}
 		else {
-			if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server)) {
+			if (pEditHandler->Remove(pData->name, pData->remotePath, pData->site)) {
 				delete pData;
 				pListCtrl->DeleteItem(i);
 			}
@@ -1307,8 +1317,9 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent&)
 void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 {
 	CEditHandler* const pEditHandler = CEditHandler::Get();
-	if (!pEditHandler)
+	if (!pEditHandler) {
 		return;
+	}
 
 	wxListCtrl* pListCtrl = XRCCTRL(*this, "ID_FILES", wxListCtrl);
 
@@ -1335,17 +1346,22 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 
 		bool unedit = event.GetId() == XRCID("ID_UPLOADANDUNEDIT") || pData->state == CEditHandler::upload_and_remove_failed;
 
-		if (type == CEditHandler::local)
+		if (type == CEditHandler::local) {
 			pEditHandler->UploadFile(pData->file, unedit);
-		else
-			pEditHandler->UploadFile(pData->name, pData->remotePath, pData->server, unedit);
+		}
+		else {
+			pEditHandler->UploadFile(pData->name, pData->remotePath, pData->site, unedit);
+		}
 
-		if (!unedit)
+		if (!unedit) {
 			pListCtrl->SetItem(i, COLUMN_STATUS, _("Uploading"));
-		else if (type == CEditHandler::remote)
+		}
+		else if (type == CEditHandler::remote) {
 			pListCtrl->SetItem(i, COLUMN_STATUS, _("Uploading and pending removal"));
-		else
+		}
+		else {
 			pListCtrl->SetItem(i, COLUMN_STATUS, _("Uploading and unediting"));
+		}
 	}
 
 	SetCtrlState();
@@ -1354,8 +1370,9 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 void CEditHandlerStatusDialog::OnEdit(wxCommandEvent&)
 {
 	CEditHandler* const pEditHandler = CEditHandler::Get();
-	if (!pEditHandler)
+	if (!pEditHandler) {
 		return;
+	}
 
 	wxListCtrl* pListCtrl = XRCCTRL(*this, "ID_FILES", wxListCtrl);
 
@@ -1391,13 +1408,14 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent&)
 			}
 		}
 		else {
-			if (!pEditHandler->StartEditing(pData->name, pData->remotePath, pData->server)) {
-				if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server)) {
+			if (!pEditHandler->StartEditing(pData->name, pData->remotePath, pData->site)) {
+				if (pEditHandler->Remove(pData->name, pData->remotePath, pData->site)) {
 					delete pData;
 					pListCtrl->DeleteItem(i);
 				}
-				else
+				else {
 					pListCtrl->SetItem(i, COLUMN_STATUS, _("Pending removal"));
+				}
 			}
 		}
 	}
@@ -1417,10 +1435,12 @@ CEditHandler::t_fileData* CEditHandlerStatusDialog::GetDataFromItem(int item, CE
 	info.SetId(item);
 	info.SetColumn(1);
 	pListCtrl->GetItem(info);
-	if (info.GetText() == _("Local"))
+	if (info.GetText() == _("Local")) {
 		type = CEditHandler::local;
-	else
+	}
+	else {
 		type = CEditHandler::remote;
+	}
 
 	return pData;
 }
@@ -1440,16 +1460,20 @@ CNewAssociationDialog::CNewAssociationDialog(wxWindow *parent)
 
 bool CNewAssociationDialog::Run(const wxString &file)
 {
-	if (!Load(m_pParent, _T("ID_EDIT_NOPROGRAM")))
+	if (!Load(m_pParent, _T("ID_EDIT_NOPROGRAM"))) {
 		return true;
+	}
 
 	int pos = file.Find('.', true);
-	if (!pos)
+	if (!pos) {
 		m_ext = _T(".");
-	else if (pos != -1)
+	}
+	else if (pos != -1) {
 		m_ext = file.Mid(pos + 1);
-	else
+	}
+	else {
 		m_ext.clear();
+	}
 
 	wxStaticText *const pDesc = XRCCTRL(*this, "ID_DESC", wxStaticText);
 	if (pDesc) {
@@ -1458,16 +1482,19 @@ bool CNewAssociationDialog::Run(const wxString &file)
 
 	bool program_exists = false;
 	wxString cmd = GetSystemOpenCommand(_T("foo.txt"), program_exists);
-	if (!program_exists)
+	if (!program_exists) {
 		cmd.clear();
+	}
 	if (!cmd.empty()) {
 		wxString args;
-		if (!UnquoteCommand(cmd, args))
+		if (!UnquoteCommand(cmd, args)) {
 			cmd.clear();
+		}
 	}
 
-	if (!PathExpand(cmd))
+	if (!PathExpand(cmd)) {
 		cmd.clear();
+	}
 
 	if (cmd.empty()) {
 		xrc_call(*this, "ID_USE_CUSTOM", &wxRadioButton::SetValue, true);
@@ -1486,8 +1513,9 @@ bool CNewAssociationDialog::Run(const wxString &file)
 
 	GetSizer()->Fit(this);
 
-	if (ShowModal() != wxID_OK)
+	if (ShowModal() != wxID_OK) {
 		return false;
+	}
 
 	return true;
 }
@@ -1529,30 +1557,36 @@ void CNewAssociationDialog::OnOK(wxCommandEvent&)
 			return;
 		}
 
-		if (always)
+		if (always) {
 			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("2") + cmd.ToStdWstring());
+		}
 		else {
 			wxString associations = COptions::Get()->GetOption(OPTION_EDIT_CUSTOMASSOCIATIONS);
-			if (!associations.empty() && associations.Last() != '\n')
+			if (!associations.empty() && associations.Last() != '\n') {
 				associations += '\n';
-			if (m_ext.empty())
+			}
+			if (m_ext.empty()) {
 				m_ext = _T("/");
+			}
 			associations += m_ext.ToStdWstring() + _T(" ") + cmd.ToStdWstring();
 			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations.ToStdWstring());
 		}
 	}
 	else {
-		if (always)
+		if (always) {
 			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("1"));
+		}
 		else {
 			bool program_exists = false;
 			wxString cmd = GetSystemOpenCommand(_T("foo.txt"), program_exists);
-			if (!program_exists)
+			if (!program_exists) {
 				cmd.clear();
+			}
 			if (!cmd.empty()) {
 				wxString args;
-				if (!UnquoteCommand(cmd, args))
+				if (!UnquoteCommand(cmd, args)) {
 					cmd.clear();
+				}
 			}
 			if (cmd.empty()
 #ifdef __WXGTK__
@@ -1590,12 +1624,14 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent&)
 #endif
 		wxFD_OPEN | wxFD_FILE_MUST_EXIST);
 
-	if (dlg.ShowModal() != wxID_OK)
+	if (dlg.ShowModal() != wxID_OK) {
 		return;
+	}
 
 	wxString editor = dlg.GetPath();
-	if (editor.empty())
+	if (editor.empty()) {
 		return;
+	}
 
 	if (!ProgramExists(editor)) {
 		XRCCTRL(*this, "ID_EDITOR", wxWindow)->SetFocus();
@@ -1603,22 +1639,23 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent&)
 		return;
 	}
 
-	if (editor.Find(' ') != -1)
+	if (editor.Find(' ') != -1) {
 		editor = _T("\"") + editor + _T("\"");
+	}
 
 	XRCCTRL(*this, "ID_CUSTOM", wxTextCtrl)->ChangeValue(editor);
 }
 
-bool CEditHandler::Edit(CEditHandler::fileType type, std::wstring const& fileName, CServerPath const& path, ServerWithCredentials const& server, int64_t size, wxWindow * parent)
+bool CEditHandler::Edit(CEditHandler::fileType type, std::wstring const& fileName, CServerPath const& path, Site const& site, int64_t size, wxWindow * parent)
 {
 	std::vector<FileData> data;
 	FileData d{fileName, size};
 	data.push_back(d);
 
-	return Edit(type, data, path, server, parent);
+	return Edit(type, data, path, site, parent);
 }
 
-bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const& data, CServerPath const& path, ServerWithCredentials const& server, wxWindow * parent)
+bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const& data, CServerPath const& path, Site const& site, wxWindow * parent)
 {
 	if (type == CEditHandler::remote) {
 		wxString const& localDir = GetLocalDirectory();
@@ -1638,14 +1675,15 @@ bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const
 		dlg.SetTitle(_("Confirmation needed"));
 		dlg.AddText(_("You have selected more than 10 files for editing, do you really want to continue?"));
 
-		if (!dlg.Run())
+		if (!dlg.Run()) {
 			return false;
+		}
 	}
 
 	bool success = true;
 	int already_editing_action{};
 	for (auto const& file : data) {
-		if (!DoEdit(type, file, path, server, parent, data.size(), already_editing_action)) {
+		if (!DoEdit(type, file, path, site, parent, data.size(), already_editing_action)) {
 			success = false;
 		}
 	}
@@ -1653,7 +1691,7 @@ bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const
 	return success;
 }
 
-bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CServerPath const& path, ServerWithCredentials const& server, wxWindow * parent, size_t fileCount, int & already_editing_action)
+bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CServerPath const& path, Site const& site, wxWindow * parent, size_t fileCount, int & already_editing_action)
 {
 	bool dangerous = false;
 	bool program_exists = false;
@@ -1683,10 +1721,12 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 	}
 
 	fileState state;
-	if (type == local)
+	if (type == local) {
 		state = GetFileState(file.name);
-	else
-		state = GetFileState(file.name, path, server);
+	}
+	else {
+		state = GetFileState(file.name, path, site);
+	}
 	switch (state)
 	{
 	case CEditHandler::download:
@@ -1696,7 +1736,7 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 		wxMessageBoxEx(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
 		return false;
 	case CEditHandler::removing:
-		if (!Remove(file.name, path, server)) {
+		if (!Remove(file.name, path, site)) {
 			wxMessageBoxEx(_("A file with that name is still being edited. Please close it and try again."), _("Selected file is already opened"), wxICON_EXCLAMATION);
 			return false;
 		}
@@ -1763,12 +1803,12 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 					StartEditing(file.name);
 				}
 				else {
-					StartEditing(file.name, path, server);
+					StartEditing(file.name, path, site);
 				}
 				return true;
 			}
 			else {
-				if (!Remove(file.name, path, server)) {
+				if (!Remove(file.name, path, site)) {
 					wxMessageBoxEx(_("The selected file is still opened in some other program, please close it."), _("Selected file is still being edited"), wxICON_EXCLAMATION);
 					return false;
 				}
@@ -1780,7 +1820,7 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 	}
 
 	std::wstring localFile = file.name;
-	if (!AddFile(type, localFile, path, server)) {
+	if (!AddFile(type, localFile, path, site)) {
 		if( type == CEditHandler::local) {
 			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), file.name), _("Opening failed"), wxICON_EXCLAMATION);
 		}
@@ -1796,7 +1836,7 @@ bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CSe
 		CLocalPath localPath(localFile, &localFileName);
 
 		m_pQueue->QueueFile(false, true, file.name, (localFileName != file.name) ? localFileName : std::wstring(),
-			localPath, path, server, file.size, type, QueuePriority::high);
+			localPath, path, site, file.size, type, QueuePriority::high);
 		m_pQueue->QueueFile_Finish(true);
 	}
 
