@@ -20,15 +20,6 @@ EVT_MENU(wxID_ANY, CQuickconnectBar::OnMenu)
 EVT_TEXT_ENTER(wxID_ANY, CQuickconnectBar::OnQuickconnect)
 END_EVENT_TABLE()
 
-CQuickconnectBar::CQuickconnectBar()
-	: m_pHost()
-	, m_pUser()
-	, m_pPass()
-	, m_pPort()
-	, m_pMainFrame()
-{
-}
-
 bool CQuickconnectBar::Create(CMainFrame* pParent)
 {
 	m_pMainFrame = pParent;
@@ -92,6 +83,15 @@ bool CQuickconnectBar::Create(CMainFrame* pParent)
 	}
 #endif
 
+	Bind(wxEVT_CHAR_HOOK, [this](wxKeyEvent& event) {
+		if (event.GetKeyCode() == WXK_DOWN) {
+			wxCommandEvent evt(wxEVT_CHAR_HOOK);
+			OnQuickconnectDropdown(evt);
+		}
+		else {
+			event.Skip();
+		}
+	});
 	return true;
 }
 
@@ -108,12 +108,12 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	std::wstring pass = m_pPass->GetValue().ToStdWstring();
 	std::wstring port = m_pPort->GetValue().ToStdWstring();
 
-	ServerWithCredentials server;
+	Site site;
 
 	std::wstring error;
 
 	CServerPath path;
-	if (!server.ParseUrl(host, port, user, pass, error, path)) {
+	if (!site.ParseUrl(host, port, user, pass, error, path)) {
 		wxString msg = _("Could not parse server address:");
 		msg += _T("\n");
 		msg += error;
@@ -121,21 +121,21 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 		return;
 	}
 
-	host = server.Format(ServerFormat::host_only);
-	ServerProtocol protocol = server.server.GetProtocol();
+	host = site.Format(ServerFormat::host_only);
+	ServerProtocol protocol = site.server.GetProtocol();
 	switch (protocol)
 	{
 	case FTP:
 	case UNKNOWN:
-		if (CServer::GetProtocolFromPort(server.server.GetPort()) != FTP &&
-			CServer::GetProtocolFromPort(server.server.GetPort()) != UNKNOWN)
+		if (CServer::GetProtocolFromPort(site.server.GetPort()) != FTP &&
+			CServer::GetProtocolFromPort(site.server.GetPort()) != UNKNOWN)
 		{
 			host = _T("ftp://") + host;
 		}
 		break;
 	default:
 		{
-			std::wstring const prefix = server.server.GetPrefixFromProtocol(protocol);
+			std::wstring const prefix = site.server.GetPrefixFromProtocol(protocol);
 			if (!prefix.empty()) {
 				host = prefix + _T("://") + host;
 			}
@@ -144,16 +144,16 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	}
 
 	m_pHost->SetValue(host);
-	if (server.server.GetPort() != server.server.GetDefaultPort(server.server.GetProtocol())) {
-		m_pPort->SetValue(wxString::Format(_T("%d"), server.server.GetPort()));
+	if (site.server.GetPort() != site.server.GetDefaultPort(site.server.GetProtocol())) {
+		m_pPort->SetValue(wxString::Format(_T("%d"), site.server.GetPort()));
 	}
 	else {
 		m_pPort->ChangeValue(wxString());
 	}
 
-	m_pUser->SetValue(server.server.GetUser());
-	if (server.credentials.logonType_ != LogonType::anonymous) {
-		m_pPass->SetValue(server.credentials.GetPass());
+	m_pUser->SetValue(site.server.GetUser());
+	if (site.credentials.logonType_ != LogonType::anonymous) {
+		m_pPass->SetValue(site.credentials.GetPass());
 	}
 	else {
 		m_pPass->ChangeValue(wxString());
@@ -166,29 +166,27 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	}
 
 	if (event.GetId() == 1) {
-		server.server.SetBypassProxy(true);
+		site.server.SetBypassProxy(true);
 	}
 
-	if (server.credentials.logonType_ != LogonType::anonymous && !CAskSavePasswordDialog::Run(this)) {
+	if (site.credentials.logonType_ != LogonType::anonymous && !CAskSavePasswordDialog::Run(this)) {
 		return;
 	}
 
-	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && server.credentials.logonType_ == LogonType::normal) {
-		server.SetLogonType(LogonType::ask);
-		CLoginManager::Get().RememberPassword(server);
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && site.credentials.logonType_ == LogonType::normal) {
+		site.SetLogonType(LogonType::ask);
+		CLoginManager::Get().RememberPassword(site);
 	}
-	Site site;
-	site.server_ = server;
 	Bookmark bm;
 	bm.m_remoteDir = path;
 	if (!m_pMainFrame->ConnectToSite(site, bm)) {
 		return;
 	}
 
-	CRecentServerList::SetMostRecentServer(server);
+	CRecentServerList::SetMostRecentServer(site);
 }
 
-void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent&)
+void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent& event)
 {
 	wxMenu* pMenu = new wxMenu;
 
@@ -208,8 +206,7 @@ void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent&)
 			iter != m_recentServers.end();
 			++iter, ++i)
 		{
-			wxString name(iter->Format(ServerFormat::with_user_and_optional_port));
-			name.Replace(_T("&"), _T("&&"));
+			wxString name(LabelEscape(iter->Format(ServerFormat::with_user_and_optional_port)));
 			pMenu->Append(10 + i, name);
 		}
 	}
@@ -217,7 +214,9 @@ void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent&)
 		pMenu->Enable(3, false);
 	}
 
-	XRCCTRL(*this, "ID_QUICKCONNECT_DROPDOWN", wxButton)->PopupMenu(pMenu);
+	auto * btn = XRCCTRL(*this, "ID_QUICKCONNECT_DROPDOWN", wxButton);
+	auto size = btn->GetSize() / 2;
+	btn->PopupMenu(pMenu, (event.GetEventType() == wxEVT_CHAR_HOOK) ? wxPoint(size.x, size.y) : wxDefaultPosition);
 	delete pMenu;
 	m_recentServers.clear();
 }
@@ -247,8 +246,7 @@ void CQuickconnectBar::OnMenu(wxCommandEvent& event)
 	auto iter = m_recentServers.cbegin();
 	std::advance(iter, index);
 
-	Site site;
-	site.server_ = *iter;
+	Site site = *iter;
 	m_pMainFrame->ConnectToSite(site, Bookmark());
 }
 
