@@ -4,27 +4,150 @@
 #include "serverdata.h"
 #include "Options.h"
 
-bool ServerWithCredentials::ParseUrl(std::wstring const& host, std::wstring const& port, std::wstring const& user, std::wstring const& pass, std::wstring &error, CServerPath &path, ServerProtocol const hint)
+bool Bookmark::operator==(Bookmark const& b) const
+{
+	if (m_localDir != b.m_localDir) {
+		return false;
+	}
+
+	if (m_remoteDir != b.m_remoteDir) {
+		return false;
+	}
+
+	if (m_sync != b.m_sync) {
+		return false;
+	}
+
+	if (m_comparison != b.m_comparison) {
+		return false;
+	}
+
+	if (m_name != b.m_name) {
+		return false;
+	}
+
+	return true;
+}
+
+Site::Site(Site const& s)
+	: server(s.server)
+	, credentials(s.credentials)
+	, comments_(s.comments_)
+	, m_default_bookmark(s.m_default_bookmark)
+	, m_bookmarks(s.m_bookmarks)
+	, m_colour(s.m_colour)
+{
+	if (s.data_) {
+		data_ = std::make_shared<SiteHandleData>(*s.data_);
+	}
+}
+
+Site& Site::operator=(Site const& s)
+{
+	if (this != &s) {
+		server = s.server;
+		credentials = s.credentials;
+		comments_ = s.comments_;
+		m_default_bookmark = s.m_default_bookmark;
+		m_bookmarks = s.m_bookmarks;
+		m_colour = s.m_colour;
+		data_.reset();
+
+		if (s.data_) {
+			data_ = std::make_shared<SiteHandleData>(*s.data_);
+		}
+	}
+
+	return *this;
+}
+
+bool Site::operator==(Site const& s) const
+{
+	if (server != s.server) {
+		return false;
+	}
+
+	if (comments_ != s.comments_) {
+		return false;
+	}
+
+	if (m_default_bookmark != s.m_default_bookmark) {
+		return false;
+	}
+
+	if (m_bookmarks != s.m_bookmarks) {
+		return false;
+	}
+
+	if (static_cast<bool>(data_) != static_cast<bool>(s.data_)) {
+		return false;
+	}
+
+	if (data_ && *data_ != *s.data_) {
+		return false;
+	}
+
+	if (m_colour != s.m_colour) {
+		return false;
+	}
+
+	return true;
+}
+
+void Site::SetSitePath(std::wstring const& sitePath) {
+	if (!data_) {
+		data_ = std::make_shared<SiteHandleData>();
+	}
+	data_->sitePath_ = sitePath;
+}
+
+std::wstring const& Site::SitePath() const
+{
+	if (data_) {
+		return data_->sitePath_;
+	}
+	else {
+		static std::wstring empty;
+		return empty;
+	}
+}
+
+ServerHandle Site::Handle() const
+{
+	return data_;
+}
+
+void Site::Update(Site const& rhs)
+{
+	std::shared_ptr<SiteHandleData> data = data_;
+	*this = rhs;
+	if (data && rhs.data_) {
+		*data = *rhs.data_;
+		data_ = data;
+	}
+}
+
+bool Site::ParseUrl(std::wstring const& host, std::wstring const& port, std::wstring const& user, std::wstring const& pass, std::wstring &error, CServerPath &path, ServerProtocol const hint)
 {
 	unsigned int nPort = 0;
 	if (!port.empty()) {
 		nPort = fz::to_integral<unsigned int>(fz::trimmed(port));
 		if (port.size() > 5 || !nPort || nPort > 65535) {
-			error = _("Invalid port given. The port has to be a value from 1 to 65535.");
+			error = fztranslate("Invalid port given. The port has to be a value from 1 to 65535.");
 			error += L"\n";
-			error += _("You can leave the port field empty to use the default port.");
+			error += fztranslate("You can leave the port field empty to use the default port.");
 			return false;
 		}
 	}
 	return ParseUrl(host, nPort, user, pass, error, path, hint);
 }
 
-bool ServerWithCredentials::ParseUrl(std::wstring host, unsigned int port, std::wstring user, std::wstring pass, std::wstring &error, CServerPath &path, ServerProtocol const hint)
+bool Site::ParseUrl(std::wstring host, unsigned int port, std::wstring user, std::wstring pass, std::wstring &error, CServerPath &path, ServerProtocol const hint)
 {
 	server.SetType(DEFAULT);
 
 	if (host.empty()) {
-		error = _("No host given, please enter a host.");
+		error = fztranslate("No host given, please enter a host.");
 		return false;
 	}
 
@@ -37,7 +160,7 @@ bool ServerWithCredentials::ParseUrl(std::wstring host, unsigned int port, std::
 		}
 		auto p = CServer::GetProtocolFromPrefix(protocol, hint);
 		if (p == UNKNOWN) {
-			error = _("Invalid protocol specified. Valid protocols are:\nftp:// for normal FTP with optional encryption,\nsftp:// for SSH file transfer protocol,\nftps:// for FTP over TLS (implicit) and\nftpes:// for FTP over TLS (explicit).");
+			error = fztranslate("Invalid protocol specified. Valid protocols are:\nftp:// for normal FTP with optional encryption,\nsftp:// for SSH file transfer protocol,\nftps:// for FTP over TLS (implicit) and\nftpes:// for FTP over TLS (explicit).");
 			return false;
 		}
 		server.SetProtocol(p);
@@ -76,7 +199,7 @@ bool ServerWithCredentials::ParseUrl(std::wstring host, unsigned int port, std::
 		fz::trim(user);
 
 		if (user.empty()) {
-			error = _("Invalid username given.");
+			error = fztranslate("Invalid username given.");
 			return false;
 		}
 	}
@@ -95,12 +218,12 @@ bool ServerWithCredentials::ParseUrl(std::wstring host, unsigned int port, std::
 		// Probably IPv6 address
 		pos = host.find(']');
 		if (pos == std::wstring::npos) {
-			error = _("Host starts with '[' but no closing bracket found.");
+			error = fztranslate("Host starts with '[' but no closing bracket found.");
 			return false;
 		}
 		if (pos < host.size() - 1) {
 			if (host[pos + 1] != ':') {
-				error = _("Invalid host, after closing bracket only colon and port may follow.");
+				error = fztranslate("Invalid host, after closing bracket only colon and port may follow.");
 				return false;
 			}
 			++pos;
@@ -114,7 +237,7 @@ bool ServerWithCredentials::ParseUrl(std::wstring host, unsigned int port, std::
 	}
 	if (pos != std::wstring::npos) {
 		if (!pos) {
-			error = _("No host given, please enter a host.");
+			error = fztranslate("No host given, please enter a host.");
 			return false;
 		}
 
@@ -128,14 +251,14 @@ bool ServerWithCredentials::ParseUrl(std::wstring host, unsigned int port, std::
 	}
 
 	if (port < 1 || port > 65535) {
-		error = _("Invalid port given. The port has to be a value from 1 to 65535.");
+		error = fztranslate("Invalid port given. The port has to be a value from 1 to 65535.");
 		return false;
 	}
 
 	fz::trim(host);
 
 	if (host.empty()) {
-		error = _("No host given, please enter a host.");
+		error = fztranslate("No host given, please enter a host.");
 		return false;
 	}
 
@@ -177,7 +300,7 @@ bool ServerWithCredentials::ParseUrl(std::wstring host, unsigned int port, std::
 	return true;
 }
 
-void ServerWithCredentials::SetLogonType(LogonType logonType)
+void Site::SetLogonType(LogonType logonType)
 {
 	credentials.logonType_ = logonType;
 	if (logonType == LogonType::anonymous) {
@@ -185,7 +308,7 @@ void ServerWithCredentials::SetLogonType(LogonType logonType)
 	}
 }
 
-void ServerWithCredentials::SetUser(std::wstring const& user)
+void Site::SetUser(std::wstring const& user)
 {
 	if (credentials.logonType_ == LogonType::anonymous) {
 		server.SetUser(L"");
@@ -210,12 +333,12 @@ void ProtectedCredentials::Protect()
 		}
 	}
 	else {
-		auto key = public_key::from_base64(fz::to_utf8(COptions::Get()->GetOption(OPTION_MASTERPASSWORDENCRYPTOR)));
+		auto key = fz::public_key::from_base64(fz::to_utf8(COptions::Get()->GetOption(OPTION_MASTERPASSWORDENCRYPTOR)));
 		Protect(key);
 	}
 }
 
-void ProtectedCredentials::Protect(public_key const& key)
+void ProtectedCredentials::Protect(fz::public_key const& key)
 {
 	if (logonType_ != LogonType::normal && logonType_ != LogonType::account) {
 		password_.clear();
@@ -257,30 +380,21 @@ void ProtectedCredentials::Protect(public_key const& key)
 	}
 }
 
-bool ProtectedCredentials::Unprotect(private_key const& key, bool on_failure_set_to_ask)
+bool ProtectedCredentials::DoUnprotect(fz::private_key const& key)
 {
-	if (!encrypted_) {
-		return true;
-	}
-
 	if (!key || key.pubkey() != encrypted_) {
-		if (on_failure_set_to_ask) {
-			encrypted_ = public_key();
-			password_.clear();
-			logonType_ = LogonType::ask;
-		}
 		return false;
 	}
 
-	auto cipher = fz::base64_decode(fz::to_utf8(password_));
+	auto const cipher = fz::base64_decode(fz::to_utf8(password_));
+
+	auto plain = fz::decrypt(cipher, key);
+	if (plain.empty()) {
+		// Compatibility with unauthenticated encryption, remove eventually.
+		plain = decrypt(cipher, key, false);
+	}
 
-	auto plain = decrypt(cipher, key);
 	if (plain.size() < 16) {
-		if (on_failure_set_to_ask) {
-			encrypted_ = public_key();
-			password_.clear();
-			logonType_ = LogonType::ask;
-		}
 		return false;
 	}
 
@@ -289,10 +403,45 @@ bool ProtectedCredentials::Unprotect(private_key const& key, bool on_failure_set
 	char const c = 0;
 	auto pos = pw.find(c);
 	if (pos != std::string::npos) {
+		if (pw.find_first_not_of(c, pos + 1) != std::string::npos) {
+			return false;
+		}
 		pw = pw.substr(0, pos);
 	}
 	password_ = fz::to_wstring_from_utf8(pw);
-	encrypted_ = public_key();
-
+	if (password_.empty() && !pw.empty()) {
+		return false;
+	}
+	encrypted_ = fz::public_key();
 	return true;
+
+}
+
+bool ProtectedCredentials::Unprotect(fz::private_key const& key, bool on_failure_set_to_ask)
+{
+	if (!encrypted_) {
+		return true;
+	}
+
+	bool const ret = DoUnprotect(key);
+	if (!ret && on_failure_set_to_ask) {
+		encrypted_ = fz::public_key();
+		password_.clear();
+		logonType_ = LogonType::ask;
+	}
+
+	return ret;
+}
+
+SiteHandleData toSiteHandle(ServerHandle const& handle)
+{
+	auto l = handle.lock();
+	if (l) {
+		auto d = dynamic_cast<SiteHandleData const*>(l.get());
+		if (d) {
+			return *d;
+		}
+	}
+
+	return SiteHandleData();
 }
