@@ -272,7 +272,7 @@ CSiteManagerDialog::~CSiteManagerDialog()
 	}
 }
 
-bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site> *connected_sites, ServerWithCredentials const* pServer)
+bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site> *connected_sites, Site const* site)
 {
 	m_pSiteManagerMutex = new CInterProcessMutex(MUTEX_SITEMANAGERGLOBAL, false);
 	if (m_pSiteManagerMutex->TryLock() == 0) {
@@ -375,8 +375,8 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site> *
 	m_connected_sites = connected_sites;
 	MarkConnectedSites();
 
-	if (pServer && *pServer) {
-		CopyAddServer(*pServer);
+	if (site && *site) {
+		CopyAddServer(*site);
 	}
 
 	return true;
@@ -391,7 +391,8 @@ void CSiteManagerDialog::MarkConnectedSites()
 
 void CSiteManagerDialog::MarkConnectedSite(int connected_site)
 {
-	std::wstring const& connected_site_path = (*m_connected_sites)[connected_site].old_path;
+	auto & site = (*m_connected_sites)[connected_site];
+	std::wstring const& connected_site_path = site.old_path;
 	if (connected_site_path.empty()) {
 		return;
 	}
@@ -403,11 +404,11 @@ void CSiteManagerDialog::MarkConnectedSite(int connected_site)
 
 	if (connected_site_path[0] == '1') {
 		// Default sites never change
-		(*m_connected_sites)[connected_site].new_path = (*m_connected_sites)[connected_site].old_path;
 		return;
 	}
 
 	if (connected_site_path[0] != '0') {
+		// Unknown type, from a future version perhaps?
 		return;
 	}
 
@@ -554,14 +555,14 @@ public:
 	virtual bool AddSite(std::unique_ptr<Site> data)
 	{
 		if (m_kiosk && !m_predefined &&
-			data->server_.credentials.logonType_ == LogonType::normal)
+			data->credentials.logonType_ == LogonType::normal)
 		{
 			// Clear saved password
-			data->server_.SetLogonType(LogonType::ask);
-			data->server_.credentials.SetPass(std::wstring());
+			data->SetLogonType(LogonType::ask);
+			data->credentials.SetPass(std::wstring());
 		}
 
-		const wxString name(data->server_.server.GetName());
+		const wxString name(data->server.GetName());
 
 		CSiteManagerItemData* pData = new CSiteManagerItemData(std::move(data));
 		wxTreeItemId newItem = m_pTree->AppendItem(m_item, name, 2, 2, pData);
@@ -808,9 +809,9 @@ bool CSiteManagerDialog::SaveChild(pugi::xml_node element, wxTreeItemId child)
 		CSiteManager::Save(node, *data->m_site);
 
 		if (data->connected_item != -1) {
-			if ((*m_connected_sites)[data->connected_item].server.server == data->m_site->server_.server) {
+			if ((*m_connected_sites)[data->connected_item].site.server == data->m_site->server) {
 				(*m_connected_sites)[data->connected_item].new_path = GetSitePath(child);
-				(*m_connected_sites)[data->connected_item].server = data->m_site->server_;
+				(*m_connected_sites)[data->connected_item].site = *data->m_site;
 			}
 		}
 	}
@@ -884,12 +885,12 @@ bool CSiteManagerDialog::Verify()
 		const wxString remotePathRaw = XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->GetValue();
 		if (!remotePathRaw.empty()) {
 			CServerPath remotePath;
-			remotePath.SetType(pServer->m_site->server_.server.GetType());
+			remotePath.SetType(pServer->m_site->server.GetType());
 			if (!remotePath.SetPath(remotePathRaw.ToStdWstring())) {
 				XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->SetFocus();
 				wxString msg;
-				if (pServer->m_site->server_.server.GetType() != DEFAULT) {
-					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the servertype (%s) selected on the parent site."), CServer::GetNameFromServerType(pServer->m_site->server_.server.GetType()));
+				if (pServer->m_site->server.GetType() != DEFAULT) {
+					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the servertype (%s) selected on the parent site."), CServer::GetNameFromServerType(pServer->m_site->server.GetType()));
 				}
 				else {
 					msg = _("Remote path cannot be parsed. Make sure it is a valid absolute path.");
@@ -1077,9 +1078,9 @@ void CSiteManagerDialog::OnNewSite(wxCommandEvent&)
 		return;
 	}
 
-	ServerWithCredentials server;
-	server.server.SetProtocol(ServerProtocol::FTP);
-	AddNewSite(item, server);
+	Site site;
+	site.server.SetProtocol(ServerProtocol::FTP);
+	AddNewSite(item, site);
 }
 
 bool CSiteManagerDialog::UpdateItem()
@@ -1114,16 +1115,16 @@ bool CSiteManagerDialog::UpdateItem()
 		if (!pServer || !pServer->m_site) {
 			return false;
 		}
-		data->m_bookmark->m_name = pTree->GetItemText(item);
-		return UpdateBookmark(*data->m_bookmark, pServer->m_site->server_);
+		data->m_bookmark->m_name = pTree->GetItemText(item).ToStdWstring();
+		return UpdateBookmark(*data->m_bookmark, *pServer->m_site);
 	}
 }
 
-bool CSiteManagerDialog::UpdateBookmark(Bookmark &bookmark, ServerWithCredentials const& server)
+bool CSiteManagerDialog::UpdateBookmark(Bookmark &bookmark, Site const& site)
 {
-	bookmark.m_localDir = xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::GetValue);
+	bookmark.m_localDir = xrc_call(*this, "ID_BOOKMARK_LOCALDIR", &wxTextCtrl::GetValue).ToStdWstring();
 	bookmark.m_remoteDir = CServerPath();
-	bookmark.m_remoteDir.SetType(server.server.GetType());
+	bookmark.m_remoteDir.SetType(site.server.GetType());
 	bookmark.m_remoteDir.SetPath(xrc_call(*this, "ID_BOOKMARK_REMOTEDIR", &wxTextCtrl::GetValue).ToStdWstring());
 	bookmark.m_sync = xrc_call(*this, "ID_BOOKMARK_SYNC", &wxCheckBox::GetValue);
 	bookmark.m_comparison = xrc_call(*this, "ID_BOOKMARK_COMPARISON", &wxCheckBox::GetValue);
@@ -1131,11 +1132,11 @@ bool CSiteManagerDialog::UpdateBookmark(Bookmark &bookmark, ServerWithCredential
 	return true;
 }
 
-void CSiteManagerDialog::UpdateServer(Site &server, const wxString &name)
+void CSiteManagerDialog::UpdateServer(Site & site, const wxString &name)
 {
-	m_pNotebook_Site->UpdateSite(server);
+	m_pNotebook_Site->UpdateSite(site);
 
-	server.server_.server.SetName(name.ToStdWstring());
+	site.server.SetName(name.ToStdWstring());
 }
 
 bool CSiteManagerDialog::GetServer(Site& data, Bookmark& bookmark)
@@ -1184,7 +1185,7 @@ bool CSiteManagerDialog::GetServer(Site& data, Bookmark& bookmark)
 		bookmark = data.m_default_bookmark;
 	}
 
-	data.m_path = GetSitePath(item);
+	data.SetSitePath(GetSitePath(item));
 
 	return true;
 }
@@ -1233,14 +1234,15 @@ void CSiteManagerDialog::SetCtrlState()
 		data = static_cast<CSiteManagerItemData*>(pTree->GetItemData(item));
 	}
 	if (!data) {
+		// Set the control states according if it's possible to use the control
+		const bool root_or_predefined = (item == pTree->GetRootItem() || item == m_ownSites || predefined);
+
 		m_pNotebook_Site->Show();
 		m_pNotebook_Bookmark->Hide();
-		m_pNotebook_Site->SetSite(Site(), predefined);
+		m_pNotebook_Site->SetSite(Site(), root_or_predefined);
 		m_pNotebook_Site->Enable(false);
 		m_pNotebook_Site->GetContainingSizer()->Layout();
 
-		// Set the control states according if it's possible to use the control
-		const bool root_or_predefined = (item == pTree->GetRootItem() || item == m_ownSites || predefined);
 
 		xrc_call(*this, "ID_RENAME", &wxWindow::Enable, !root_or_predefined);
 		xrc_call(*this, "ID_DELETE", &wxWindow::Enable, !root_or_predefined);
@@ -1647,13 +1649,13 @@ void CSiteManagerDialog::OnChar(wxKeyEvent& event)
 	OnRename(cmdEvent);
 }
 
-void CSiteManagerDialog::CopyAddServer(ServerWithCredentials const& server)
+void CSiteManagerDialog::CopyAddServer(Site const& site)
 {
 	if (!Verify()) {
 		return;
 	}
 
-	AddNewSite(m_ownSites, server, true);
+	AddNewSite(m_ownSites, site, true);
 }
 
 wxString CSiteManagerDialog::FindFirstFreeName(const wxTreeItemId &parent, const wxString& name)
@@ -1687,7 +1689,7 @@ wxString CSiteManagerDialog::FindFirstFreeName(const wxTreeItemId &parent, const
 	return newName;
 }
 
-void CSiteManagerDialog::AddNewSite(wxTreeItemId parent, ServerWithCredentials const& server, bool connected)
+void CSiteManagerDialog::AddNewSite(wxTreeItemId parent, Site const& site, bool connected)
 {
 	wxTreeCtrlEx *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrlEx);
 	if (!pTree) {
@@ -1698,7 +1700,7 @@ void CSiteManagerDialog::AddNewSite(wxTreeItemId parent, ServerWithCredentials c
 
 	CSiteManagerItemData* pData = new CSiteManagerItemData;
 	pData->m_site = std::make_unique<Site>();
-	pData->m_site->server_ = server;
+	*pData->m_site = site;
 	if (connected) {
 		pData->connected_item = 0;
 	}
