@@ -1,7 +1,9 @@
 #include "filezilla.h"
-
+#include "filezillaapp.h"
 #include "xrc_helper.h"
 
+#include <libfilezilla/local_filesys.hpp>
+
 #include <wx/xrc/xh_animatctrl.h>
 #include <wx/xrc/xh_bmpbt.h>
 #include <wx/xrc/xh_bttn.h>
@@ -53,3 +55,31 @@ void InitHandlers(wxXmlResource& res)
 	res.AddHandler(new wxHyperlinkCtrlXmlHandler);
 	res.AddHandler(new wxAnimationCtrlXmlHandler);
 }
+
+void InitXrc()
+{
+	static bool initialized = false;
+	if (initialized) {
+		return;
+	}
+	initialized = true;
+
+	wxXmlResource *pResource = wxXmlResource::Get();
+
+#ifndef __WXDEBUG__
+	pResource->SetFlags(pResource->GetFlags() | wxXRC_NO_RELOADING);
+#endif
+
+	InitHandlers(*pResource);
+
+	fz::local_filesys fs;
+	std::wstring dir = wxGetApp().GetResourceDir().GetPath() + L"xrc/";
+	pResource->LoadFile(wxString(dir + L"certificate.xrc"));
+	pResource->LoadFile(wxString(dir + L"dialogs.xrc"));
+	pResource->LoadFile(wxString(dir + L"inputdialog.xrc"));
+	pResource->LoadFile(wxString(dir + L"netconfwizard.xrc"));
+	pResource->LoadFile(wxString(dir + L"settings.xrc"));
+	pResource->LoadFile(wxString(dir + L"sitemanager.xrc"));
+	pResource->LoadFile(wxString(dir + L"update.xrc"));
+	pResource->LoadFile(wxString(dir + L"storj.xrc"));
+}
